package speiger.src.coreengine.rendering.input.devices;

import java.nio.ByteBuffer;
import java.nio.FloatBuffer;
import java.util.BitSet;

import org.lwjgl.glfw.GLFW;
import org.lwjgl.system.Callback;

import speiger.src.collections.floats.lists.FloatArrayList;
import speiger.src.collections.floats.lists.FloatList;
import speiger.src.collections.ints.sets.IntLinkedOpenHashSet;
import speiger.src.collections.ints.sets.IntSet;
import speiger.src.collections.longs.collections.LongIterator;
import speiger.src.coreengine.rendering.input.devices.Joystick.JoyStickData;
import speiger.src.coreengine.rendering.input.devices.Joystick.JoyStickTask;
import speiger.src.coreengine.utils.eventbus.EventBus;

public class Joystick extends AbstractDevice<JoyStickData, JoyStickTask> {
	public static final Joystick INSTANCE = new Joystick();
	IntSet presentJoysticks = new IntLinkedOpenHashSet();
	Callback callback;
	
	@Override
	public void init(EventBus bus) {
		super.init(bus);
		callback = GLFW.glfwSetJoystickCallback(this::plugin);
		for(int i = 0,m=GLFW.GLFW_JOYSTICK_LAST;i<=m;i++) {
			if(GLFW.glfwJoystickPresent(i)) {
				presentJoysticks.add(i);
			}
		}
	}
	
	public void destroy() {
		callback.free();
	}
	
	private void plugin(int jid, int event) {
		if(event == GLFW.GLFW_CONNECTED) presentJoysticks.add(jid);
		else presentJoysticks.remove(jid);
		for(LongIterator iter = knownWindows().iterator();iter.hasNext();) {
			long window = iter.nextLong();
			push(window, new Plugin(window, jid, event));
		}
	}
	
	@Override
	public void reset(long windowId) {}
	@Override
	protected JoyStickData createData(long windowId) { return new JoyStickData(); }
	@Override
	protected void process(JoyStickTask task) { task.process(this); }
	
	@Override
	public void processInput(long windowId) {
		super.processInput(windowId);
	}
	
	public void handleJoystick(int jid, ButtonData data) {
		ByteBuffer buttons = GLFW.glfwGetJoystickButtons(jid);
		for(int i = 0;buttons.hasRemaining();i++) {
			data.buttons.set(i, buttons.get() == GLFW.GLFW_PRESS);
		}
		FloatBuffer axis = GLFW.glfwGetJoystickAxes(jid);
		for(int i = 0;axis.hasRemaining();i++) {
			float state = axis.get();
			if(i >= data.axis.size()) data.axis.add(state);
			else data.axis.set(i, state);
			boolean max = state < -0.95F || state > 0.95F;
			boolean min = state < -0.5F || state > 0.5F;
			data.axisState.set(i*3, state < 0);
			data.axisState.set(i*3+1, max);
			data.axisState.set(i*3+2, !max && min);				
		}
	}
	
	public static class JoyStickData {
		
	}
	
	public static class ButtonData {
		BitSet buttons = new BitSet();
		FloatList axis = new FloatArrayList();
		BitSet axisState = new BitSet();
	}
	
	public static interface JoyStickTask {
		public void process(Joystick stick);
	}
	
	public record Plugin(long window, int jid, int event) implements JoyStickTask {
		@Override
		public void process(Joystick stick) {
			
		}
	}
}
